@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
 
 Copyright (C) 2014  Vyacheslav Trushkin
-Copyright (C) 2020-2025  Boian Bonev
+Copyright (C) 2020-2026  Boian Bonev
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 
@@ -27,7 +27,7 @@ inline int system_checks(void) {
 	int io_accounting=0;
 	uint64_t i,o;
 
-	if (!is_a_process(1)) {
+	if (!is_a_process(getpid())) {
 		printf("Looks like /proc is not mounted.\n");
 		return 1;
 	}
