@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
 
 Copyright (C) 2014  Vyacheslav Trushkin
-Copyright (C) 2020-2025  Boian Bonev
+Copyright (C) 2020-2026  Boian Bonev
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 
@@ -24,11 +24,15 @@ You should have received a copy of the GNU General Public License along with thi
 #include <sys/stat.h>
 #include <sys/types.h>
 
-inline char *read_cmdline(int pid,int isshort) {
-	char *rv=NULL;
+inline void read_cmdlines(int pid,char **cmd_long,char **cmd_short,char **cmd_comm) {
 	char path[30];
 	int fd;
 
+	if (!cmd_long||!cmd_short||!cmd_comm)
+		return;
+
+	*cmd_long=*cmd_short=*cmd_comm=NULL;
+
 	snprintf(path,sizeof path,"/proc/%d/cmdline",pid);
 	fd=open(path,O_RDONLY);
 	if (fd!=-1) {
@@ -37,7 +41,7 @@ inline char *read_cmdline(int pid,int isshort) {
 
 		if (!dbuf) {
 			close(fd);
-			return NULL;
+			return;
 		}
 
 		do {
@@ -48,7 +52,62 @@ inline char *read_cmdline(int pid,int isshort) {
 				if (!t) {
 					close(fd);
 					free(dbuf);
-					return NULL;
+					return;
+				}
+				dbuf=t;
+				sz+=BUFSIZ;
+			}
+			if (n>0)
+				p+=n;
+		} while (n>0);
+
+		if (p>0) {
+			ssize_t k;
+			dbuf[p]=0;
+
+			*cmd_short=strdup(dbuf);
+			if (*cmd_short) {
+				char *ep;
+
+				ep=strrchr(*cmd_short,'/');
+				if (ep&&ep[1]) {
+					char *t=strdup(ep+1);
+
+					if (t) {
+						free(*cmd_short);
+						*cmd_short=t;
+					}
+				}
+			}
+
+			for (k=0;k<p;k++)
+				dbuf[k]=dbuf[k]?dbuf[k]:' ';
+			*cmd_long=dbuf;
+		} else
+			free(dbuf);
+		close(fd);
+	}
+
+	snprintf(path,sizeof path,"/proc/%d/comm",pid);
+	fd=open(path,O_RDONLY);
+	if (fd!=-1) {
+		char *dbuf=malloc(BUFSIZ+1);
+		ssize_t n,p=0,sz=BUFSIZ;
+
+		if (!dbuf) {
+			close(fd);
+			return;
+		}
+
+		do {
+			n=read(fd,dbuf+p,sz-p);
+			if (n==sz-p) {
+				char *t=realloc(dbuf,sz+BUFSIZ+1);
+
+				if (!t) {
+					close(fd);
+					free(dbuf);
+					return;
 				}
 				dbuf=t;
 				sz+=BUFSIZ;
@@ -59,63 +118,29 @@ inline char *read_cmdline(int pid,int isshort) {
 
 		if (p>0) {
 			dbuf[p]=0;
-			if (isshort&&(dbuf[0]=='/'||(p>1&&dbuf[0]=='.'&&dbuf[1]=='/')||(p>2&&dbuf[0]=='.'&&dbuf[1]=='.'&&dbuf[2]=='/'))) {
-				char *ep;
 
-				ep=strrchr(dbuf,'/');
-				if (ep&&ep[1]) {
-					char *t=strdup(ep+1);
+			while (strlen(dbuf)&&dbuf[strlen(dbuf)-1]=='\n') // trim newline(s) from the end
+				dbuf[strlen(dbuf)-1]=0;
 
-					if (t) {
+			if (*cmd_short&&strcmp(*cmd_short,dbuf))
+				*cmd_comm=dbuf; // thread with different name
+			else {
+				if (*cmd_short) // short cmdline matches comm, discard cmd_comm
+					free(dbuf);
+				else { // if there is no memory alloc error, both cmd_long and cmd_short should be NULL or non-NULL
+					*cmd_short=malloc(strlen(dbuf)+3);
+					if (*cmd_short)
+						sprintf(*cmd_short,"[%s]",dbuf);
+					if (!*cmd_long)
+						*cmd_long=dbuf;
+					else
 						free(dbuf);
-						dbuf=t;
-						p=strlen(t)+1;
-					}
 				}
 			}
-
-			if (!isshort) {
-				ssize_t k;
-
-				for (k=0;k<p;k++)
-					dbuf[k]=dbuf[k]?dbuf[k]:' ';
-			}
-			rv=dbuf;
 		} else
 			free(dbuf);
 		close(fd);
 	}
-
-	if (rv)
-		return rv;
-
-	snprintf(path,sizeof path,"/proc/%d/status",pid);
-	fd=open(path,O_RDONLY);
-	if (fd!=-1) {
-		char buf[BUFSIZ+1];
-		ssize_t n=read(fd,buf,BUFSIZ);
-
-		close(fd);
-
-		if (n>0) {
-			char *eol,*tab;
-
-			buf[n]=0;
-			eol=strchr(buf,'\n');
-			tab=strchr(buf,'\t');
-			if (eol&&tab&&eol>tab) {
-				size_t rvlen;
-
-				eol[0]=0;
-				rvlen=strlen(tab+1)+3;
-				rv=malloc(rvlen);
-				if (rv)
-					snprintf(rv,rvlen,!isshort?"[%s]":"%s",tab+1);
-			}
-		}
-	}
-
-	return rv;
 }
 
 inline void pidgen_cb(pg_cb cb,struct xxxid_stats_arr *hint1,filter_callback hint2) {
@@ -340,10 +365,10 @@ inline int is_a_dir(const char *p) {
 }
 
 inline int is_a_process(pid_t tid) {
-	char path[30];
+	char path[35];
 
-	snprintf(path,sizeof path,"/proc/%d",tid);
-	return is_a_dir(path);
+	snprintf(path,sizeof path,"/proc/%d/stat",tid);
+	return is_a_file(path);
 }
 
 inline double timediff_in_s(uint64_t sta,uint64_t end) {
