@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
 
 Copyright (C) 2014  Vyacheslav Trushkin
-Copyright (C) 2020-2025  Boian Bonev
+Copyright (C) 2020-2026  Boian Bonev
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 
@@ -25,6 +25,13 @@ You should have received a copy of the GNU General Public License along with thi
 #include <linux/taskstats.h>
 #include <linux/genetlink.h>
 
+// Recent Linux kernel broke the backwards compatibility in struct taskstats,
+// rendering the kernel headers useless and forcing userland to make hacks
+// like the one below... :(
+
+#include "taskstats-v14.h"
+#include "taskstats-v15.h"
+
 /*
  * Generic macros for dealing with netlink sockets. Might be duplicated
  * elsewhere
@@ -184,15 +191,33 @@ inline int nl_xxxid_info(pid_t tid,pid_t pid,struct xxxid_stats *stats) {
 			na=(struct nlattr *)NLA_DATA(na);
 			while (len2<aggr_len) {
 				if (na->nla_type==TASKSTATS_TYPE_STATS) {
+					// NOTE: we use the build system kernel headers for the version field only
+					// all the data access is done by using copies of the respective versions
+					// of the kernel headers
+					// A patch that will fix the problem is proposed to be included in the kernel
+					// and the only broken struct taskstats will be the one with v15. But we can
+					// not rely on the build system kernel headers for universal access and have
+					// to keep the copies.
+					// In this way a build with any kernel headers will work everywhere
 					struct taskstats *ts=NLA_DATA(na);
+					struct taskstats_v14 *t14=NLA_DATA(na);
+					struct taskstats_v15 *t15=NLA_DATA(na);
 
-					#define COPY(field) { stats->field = ts->field; }
-					COPY(read_bytes);
-					COPY(write_bytes);
-					COPY(swapin_delay_total);
-					COPY(blkio_delay_total);
-					#undef COPY
-					stats->euid=ts->ac_uid;
+					if (ts->version<IOTOP_TASKSTATS_MINVER) // v3 and below does not have the data we require
+						taskstats_ver=ts->version;
+					else if (ts->version!=15) { // use v14 for v4..v14 & v16 onwards
+						stats->read_bytes=t14->read_bytes;
+						stats->write_bytes=t14->write_bytes;
+						stats->swapin_delay_total=t14->swapin_delay_total;
+						stats->blkio_delay_total=t14->blkio_delay_total;
+						stats->euid=t14->ac_uid;
+					} else { // exception for v15 only
+						stats->read_bytes=t15->read_bytes;
+						stats->write_bytes=t15->write_bytes;
+						stats->swapin_delay_total=t15->swapin_delay_total;
+						stats->blkio_delay_total=t15->blkio_delay_total;
+						stats->euid=t15->ac_uid;
+					}
 				}
 				len2+=NLA_ALIGN(na->nla_len);
 				na=(struct nlattr *)((char *)na+len2);
@@ -210,10 +235,12 @@ inline void nl_fini(void) {
 }
 
 inline void free_stats(struct xxxid_stats *s) {
-	if (s->cmdline1)
-		free(s->cmdline1);
-	if (s->cmdline2)
-		free(s->cmdline2);
+	if (s->cmdline_short)
+		free(s->cmdline_short);
+	if (s->cmdline_long)
+		free(s->cmdline_long);
+	if (s->cmdline_comm)
+		free(s->cmdline_comm);
 	if (s->pw_name)
 		free(s->pw_name);
 	arr_free_noitem(s->threads);
@@ -222,13 +249,15 @@ inline void free_stats(struct xxxid_stats *s) {
 }
 
 inline struct xxxid_stats *make_stats(pid_t tid,pid_t pid) {
-	struct xxxid_stats *s=calloc(1,sizeof *s);
 	static const char unknown[]="<unknown>";
+	struct xxxid_stats *s;
 	struct passwd *pwd;
-	char *cmdline1;
-	char *cmdline2;
 	int prio;
 
+	if (!is_a_process(tid))
+		return NULL;
+
+	s=calloc(1,sizeof *s);
 	if (!s)
 		return NULL;
 
@@ -243,15 +272,17 @@ inline struct xxxid_stats *make_stats(pid_t tid,pid_t pid) {
 	} else
 		s->io_prio=prio;
 
-	cmdline1=read_cmdline(tid,1);
-	cmdline2=read_cmdline(tid,0);
+	read_cmdlines(tid,&s->cmdline_long,&s->cmdline_short,&s->cmdline_comm);
 
-	s->cmdline1=cmdline1?cmdline1:strdup(unknown);
-	s->cmdline2=cmdline2?cmdline2:strdup(unknown);
+	if (!s->cmdline_long)
+		s->cmdline_long=strdup(unknown);
+	if (!s->cmdline_short)
+		s->cmdline_short=strdup(unknown);
+	// cmdline_comm can be NULL
 	pwd=getpwuid(s->euid);
 	s->pw_name=strdup(pwd&&pwd->pw_name?pwd->pw_name:unknown);
 
-	if ((s->error_x||s->error_i||!cmdline1||!cmdline2)&&!is_a_process(tid)) { // process exited in the meantime
+	if ((s->error_x||s->error_i||!s->cmdline_long||!s->cmdline_short)&&!is_a_process(tid)) { // process exited in the meantime
 		free_stats(s);
 		return NULL;
 	}
